import pytest
from flask import Flask
from flask import jsonify

from flask_jwt_extended import create_access_token
from flask_jwt_extended import current_user
from flask_jwt_extended import get_current_user
from flask_jwt_extended import jwt_required
from flask_jwt_extended import JWTManager
from tests.utils import get_jwt_manager
from tests.utils import make_headers


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    JWTManager(app)

    @app.route("/get_user1", methods=["GET"])
    @jwt_required()
    def get_user1():
        try:
            return jsonify(foo=get_current_user()["username"])
        except RuntimeError as e:
            return jsonify(error=str(e))

    @app.route("/get_user2", methods=["GET"])
    @jwt_required()
    def get_user2():
        try:
            return jsonify(foo=current_user["username"])
        except RuntimeError as e:
            return jsonify(error=str(e))

    return app


@pytest.mark.parametrize("url", ["/get_user1", "/get_user2"])
def test_no_user_lookup_loader_specified(app, url):
    test_client = app.test_client()
    with app.test_request_context():
        access_token = create_access_token("username")

    response = test_client.get(url, headers=make_headers(access_token))
    assert "@jwt.user_lookup_loader" in response.get_json()["error"]


@pytest.mark.parametrize("url", ["/get_user1", "/get_user2"])
def test_load_valid_user(app, url):
    jwt = get_jwt_manager(app)

    @jwt.user_lookup_loader
    def user_lookup_callback(_jwt_header, jwt_data):
        return {"username": jwt_data["sub"]}

    test_client = app.test_client()
    with app.test_request_context():
        access_token = create_access_token("username")

    response = test_client.get(url, headers=make_headers(access_token))
    assert response.status_code == 200
    assert response.get_json() == {"foo": "username"}


@pytest.mark.parametrize("url", ["/get_user1", "/get_user2"])
def test_load_invalid_user(app, url):
    jwt = get_jwt_manager(app)

    @jwt.user_lookup_loader
    def user_lookup_callback(_jwt_header, jwt_data):
        return None

    test_client = app.test_client()
    with app.test_request_context():
        access_token = create_access_token("username")

    response = test_client.get(url, headers=make_headers(access_token))
    assert response.status_code == 401
    assert response.get_json() == {"msg": "Error loading the user username"}


@pytest.mark.parametrize("url", ["/get_user1", "/get_user2"])
def test_custom_user_lookup_errors(app, url):
    jwt = get_jwt_manager(app)

    @jwt.user_lookup_loader
    def user_lookup_callback(_jwt_header, jwt_data):
        return None

    @jwt.user_lookup_error_loader
    def user_lookup_error(jwt_header, jwt_data):
        assert jwt_header["alg"] == "HS256"
        assert jwt_data["sub"] == "username"
        return jsonify(foo="bar"), 201

    test_client = app.test_client()
    with app.test_request_context():
        access_token = create_access_token("username")

    response = test_client.get(url, headers=make_headers(access_token))
    assert response.status_code == 201
    assert response.get_json() == {"foo": "bar"}



import pytest
from flask import Flask
from flask import jsonify

from flask_jwt_extended import create_access_token
from flask_jwt_extended import jwt_required
from flask_jwt_extended import JWTManager
from tests.utils import get_jwt_manager


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    app.config["JWT_TOKEN_LOCATION"] = ["query_string"]
    JWTManager(app)

    @app.route("/protected", methods=["GET"])
    @jwt_required()
    def access_protected():
        return jsonify(foo="bar")

    return app


def test_default_query_paramater(app):
    test_client = app.test_client()

    with app.test_request_context():
        access_token = create_access_token("username")

    url = "/protected?jwt={}".format(access_token)
    response = test_client.get(url)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


def test_query_string_value_prefix(app):
    app.config["JWT_QUERY_STRING_VALUE_PREFIX"] = "bearer "
    test_client = app.test_client()

    with app.test_request_context():
        access_token = create_access_token("username")

    # Valid string prefix
    url = f"/protected?jwt=bearer {access_token}"
    response = test_client.get(url)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}

    # Invalid string prefix
    url = f"/protected?jwt={access_token}"
    response = test_client.get(url)
    error_msg = (
        "Invalid value for query parameter 'jwt'. "
        "Expected the value to start with 'bearer '"
    )
    assert response.status_code == 422
    assert response.get_json() == {"msg": error_msg}


def test_custom_query_paramater(app):
    app.config["JWT_QUERY_STRING_NAME"] = "foo"
    test_client = app.test_client()

    with app.test_request_context():
        access_token = create_access_token("username")

    # Insure 'default' query paramaters no longer work
    url = "/protected?jwt={}".format(access_token)
    response = test_client.get(url)
    assert response.status_code == 401
    assert response.get_json() == {"msg": "Missing 'foo' query paramater"}

    # Insure new query_string does work
    url = "/protected?foo={}".format(access_token)
    response = test_client.get(url)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


def test_missing_query_paramater(app):
    test_client = app.test_client()
    jwtM = get_jwt_manager(app)

    with app.test_request_context():
        access_token = create_access_token("username")

    # Insure no query paramaters doesn't give a response
    response = test_client.get("/protected")
    assert response.status_code == 401
    assert response.get_json() == {"msg": "Missing 'jwt' query paramater"}

    # Insure headers don't work
    access_headers = {"Authorization": "Bearer {}".format(access_token)}
    response = test_client.get("/protected", headers=access_headers)
    assert response.status_code == 401
    assert response.get_json() == {"msg": "Missing 'jwt' query paramater"}

    # Test custom response works
    @jwtM.unauthorized_loader
    def custom_response(err_str):
        return jsonify(foo="bar"), 201

    response = test_client.get("/protected")
    assert response.status_code == 201
    assert response.get_json() == {"foo": "bar"}



import pytest
from flask import Flask
from flask import jsonify

from flask_jwt_extended import create_access_token
from flask_jwt_extended import get_jwt_request_location
from flask_jwt_extended import jwt_required
from flask_jwt_extended import JWTManager
from flask_jwt_extended import set_access_cookies


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    app.config["JWT_TOKEN_LOCATION"] = ["headers", "cookies", "query_string", "json"]
    JWTManager(app)

    @app.route("/cookie_login", methods=["GET"])
    def cookie_login():
        resp = jsonify(login=True)
        access_token = create_access_token("username")
        set_access_cookies(resp, access_token)
        return resp

    @app.route("/protected", methods=["GET", "POST"])
    @jwt_required()
    def access_protected():
        return jsonify(foo="bar", location=get_jwt_request_location())

    return app


@pytest.fixture(scope="function")
def app_with_locations():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    app.config["JWT_TOKEN_LOCATION"] = ["headers"]
    locations = ["headers", "cookies", "query_string", "json"]
    JWTManager(app)

    @app.route("/cookie_login", methods=["GET"])
    def cookie_login():
        resp = jsonify(login=True)
        access_token = create_access_token("username")
        set_access_cookies(resp, access_token)
        return resp

    @app.route("/protected", methods=["GET", "POST"])
    @jwt_required(locations=locations)
    def access_protected():
        return jsonify(foo="bar", location=get_jwt_request_location())

    return app


def test_header_access(app, app_with_locations):
    for app in (app, app_with_locations):
        test_client = app.test_client()
        with app.test_request_context():
            access_token = create_access_token("username")

        access_headers = {"Authorization": "Bearer {}".format(access_token)}
        response = test_client.get("/protected", headers=access_headers)
        assert response.status_code == 200
        assert response.get_json() == {"foo": "bar", "location": "headers"}


def test_cookie_access(app, app_with_locations):
    for app in (app, app_with_locations):
        test_client = app.test_client()
        test_client.get("/cookie_login")
        response = test_client.get("/protected")
        assert response.status_code == 200
        assert response.get_json() == {"foo": "bar", "location": "cookies"}


def test_query_string_access(app, app_with_locations):
    for app in (app, app_with_locations):
        test_client = app.test_client()
        with app.test_request_context():
            access_token = create_access_token("username")

        url = "/protected?jwt={}".format(access_token)
        response = test_client.get(url)
        assert response.status_code == 200
        assert response.get_json() == {"foo": "bar", "location": "query_string"}


def test_json_access(app, app_with_locations):
    for app in (app, app_with_locations):
        test_client = app.test_client()
        with app.test_request_context():
            access_token = create_access_token("username")
        data = {"access_token": access_token}
        response = test_client.post("/protected", json=data)
        assert response.status_code == 200
        assert response.get_json() == {"foo": "bar", "location": "json"}


@pytest.mark.parametrize(
    "options",
    [
        (
            ["cookies", "headers"],
            (
                "Missing JWT in cookies or headers (Missing cookie "
                '"access_token_cookie"; Missing Authorization Header)'
            ),
        ),
        (
            ["json", "query_string"],
            (
                "Missing JWT in json or query_string (Invalid "
                "content-type. Must be application/json.; "
                "Missing 'jwt' query paramater)"
            ),
        ),
    ],
)
def test_no_jwt_in_request(app, options):
    token_locations, expected_err = options
    app.config["JWT_TOKEN_LOCATION"] = token_locations
    test_client = app.test_client()
    response = test_client.get("/protected")
    assert response.status_code == 401
    assert response.get_json() == {"msg": expected_err}


@pytest.mark.parametrize(
    "options",
    [
        (["cookies", "headers"], 200, None, {"foo": "bar", "location": "cookies"}),
        (["headers", "cookies"], 200, None, {"foo": "bar", "location": "cookies"}),
    ],
)
def test_order_of_jwt_locations_in_request(app, options):
    """ test order doesn't matter if at least one valid token is set"""
    token_locations, status_code, expected_err, expected_dict = options
    app.config["JWT_TOKEN_LOCATION"] = token_locations
    test_client = app.test_client()
    test_client.get("/cookie_login")
    response = test_client.get("/protected")

    assert response.status_code == status_code
    if expected_dict:
        assert response.get_json() == expected_dict
    else:
        assert response.get_json() == {"msg": expected_err}


@pytest.mark.parametrize(
    "options",
    [
        (["cookies", "headers"], 200, None, {"foo": "bar", "location": "cookies"}),
        (["headers", "cookies"], 422, ("Invalid header padding"), None),
    ],
)
def test_order_of_jwt_locations_with_one_invalid_token_in_request(app, options):
    """ test order doesn't matter if at least one valid token is set"""
    token_locations, status_code, expected_err, expected_dict = options
    app.config["JWT_TOKEN_LOCATION"] = token_locations
    test_client = app.test_client()

    with app.test_request_context():
        access_token = create_access_token("username")
    # invalidate the token, to check token location precedence
    access_token = "000000{}".format(access_token[5:])
    access_headers = {"Authorization": "Bearer {}".format(access_token)}
    # set valid cookies
    test_client.get("/cookie_login")
    response = test_client.get("/protected", headers=access_headers)

    assert response.status_code == status_code
    if expected_dict:
        assert response.get_json() == expected_dict
    else:
        assert response.get_json() == {"msg": expected_err}



import pytest
from flask import Flask
from flask import jsonify

from flask_jwt_extended import create_access_token
from flask_jwt_extended import create_refresh_token
from flask_jwt_extended import jwt_required
from flask_jwt_extended import JWTManager
from flask_jwt_extended import set_access_cookies
from flask_jwt_extended import set_refresh_cookies
from flask_jwt_extended import unset_access_cookies
from flask_jwt_extended import unset_jwt_cookies
from flask_jwt_extended import unset_refresh_cookies


def _get_cookie_from_response(response, cookie_name):
    cookie_headers = response.headers.getlist("Set-Cookie")
    for header in cookie_headers:
        attributes = header.split(";")
        if cookie_name in attributes[0]:
            cookie = {}
            for attr in attributes:
                split = attr.split("=")
                cookie[split[0].strip().lower()] = split[1] if len(split) > 1 else True
            return cookie
    return None


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    app.config["JWT_TOKEN_LOCATION"] = ["cookies"]
    JWTManager(app)

    @app.route("/access_token", methods=["GET"])
    def access_token():
        resp = jsonify(login=True)
        access_token = create_access_token("username")
        set_access_cookies(resp, access_token)
        return resp

    @app.route("/refresh_token", methods=["GET"])
    def refresh_token():
        resp = jsonify(login=True)
        refresh_token = create_refresh_token("username")
        set_refresh_cookies(resp, refresh_token)
        return resp

    @app.route("/delete_tokens", methods=["GET"])
    def delete_tokens():
        resp = jsonify(logout=True)
        unset_jwt_cookies(resp)
        return resp

    @app.route("/delete_access_tokens", methods=["GET"])
    def delete_access_tokens():
        resp = jsonify(access_revoked=True)
        unset_access_cookies(resp)
        return resp

    @app.route("/delete_refresh_tokens", methods=["GET"])
    def delete_refresh_tokens():
        resp = jsonify(refresh_revoked=True)
        unset_refresh_cookies(resp)
        return resp

    @app.route("/protected", methods=["GET"])
    @jwt_required()
    def protected():
        return jsonify(foo="bar")

    @app.route("/post_protected", methods=["POST"])
    @jwt_required()
    def post_protected():
        return jsonify(foo="bar")

    @app.route("/refresh_protected", methods=["GET"])
    @jwt_required(refresh=True)
    def refresh_protected():
        return jsonify(foo="bar")

    @app.route("/post_refresh_protected", methods=["POST"])
    @jwt_required(refresh=True)
    def post_refresh_protected():
        return jsonify(foo="bar")

    @app.route("/optional_post_protected", methods=["POST"])
    @jwt_required(optional=True)
    def optional_post_protected():
        return jsonify(foo="bar")

    return app


@pytest.mark.parametrize(
    "options",
    [
        (
            "/refresh_token",
            "refresh_token_cookie",
            "/refresh_protected",
            "/delete_refresh_tokens",
        ),  # nopep8
        ("/access_token", "access_token_cookie", "/protected", "/delete_access_tokens"),
    ],
)
def test_jwt_refresh_required_with_cookies(app, options):
    test_client = app.test_client()
    auth_url, cookie_name, protected_url, delete_url = options

    # Test without cookies
    response = test_client.get(protected_url)
    assert response.status_code == 401
    assert response.get_json() == {"msg": 'Missing cookie "{}"'.format(cookie_name)}

    # Test after receiving cookies
    test_client.get(auth_url)
    response = test_client.get(protected_url)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}

    # Test after issuing a 'logout' to delete the cookies
    test_client.get(delete_url)
    response = test_client.get(protected_url)
    assert response.status_code == 401
    assert response.get_json() == {"msg": 'Missing cookie "{}"'.format(cookie_name)}

    # log back in once more to test that clearing all tokens works
    test_client.get(auth_url)
    response = test_client.get(protected_url)
    assert response.status_code == 200

    test_client.get("/delete_tokens")
    response = test_client.get(protected_url)
    assert response.status_code == 401
    assert response.get_json() == {"msg": 'Missing cookie "{}"'.format(cookie_name)}


@pytest.mark.parametrize(
    "options",
    [
        ("/refresh_token", "csrf_refresh_token", "/post_refresh_protected"),
        ("/access_token", "csrf_access_token", "/post_protected"),
    ],
)
def test_default_access_csrf_protection(app, options):
    test_client = app.test_client()
    auth_url, csrf_cookie_name, post_url = options

    # Get the jwt cookies and csrf double submit tokens
    response = test_client.get(auth_url)
    csrf_token = _get_cookie_from_response(response, csrf_cookie_name)[csrf_cookie_name]

    # Test you cannot post without the additional csrf protection
    response = test_client.post(post_url)
    assert response.status_code == 401
    assert response.get_json() == {"msg": "Missing CSRF token"}

    # Test that you can post with the csrf double submit value
    csrf_headers = {"X-CSRF-TOKEN": csrf_token}
    response = test_client.post(post_url, headers=csrf_headers)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


@pytest.mark.parametrize(
    "options",
    [
        ("/refresh_token", "/post_refresh_protected"),
        ("/access_token", "/post_protected"),
    ],
)
def test_non_matching_csrf_token(app, options):
    test_client = app.test_client()
    auth_url, post_url = options

    # Get the jwt cookies and csrf double submit tokens
    test_client.get(auth_url)
    csrf_headers = {"X-CSRF-TOKEN": "totally_wrong_token"}
    response = test_client.post(post_url, headers=csrf_headers)
    assert response.status_code == 401
    assert response.get_json() == {"msg": "CSRF double submit tokens do not match"}


@pytest.mark.parametrize(
    "options",
    [
        ("/refresh_token", "/post_refresh_protected"),
        ("/access_token", "/post_protected"),
    ],
)
def test_csrf_disabled(app, options):
    app.config["JWT_COOKIE_CSRF_PROTECT"] = False
    test_client = app.test_client()
    auth_url, post_url = options

    # Get the jwt cookies and csrf double submit tokens
    test_client.get(auth_url)
    response = test_client.post(post_url)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


@pytest.mark.parametrize(
    "options",
    [
        ("/refresh_token", "csrf_refresh_token", "/post_refresh_protected"),
        ("/access_token", "csrf_access_token", "/post_protected"),
    ],
)
def test_csrf_with_custom_header_names(app, options):
    app.config["JWT_ACCESS_CSRF_HEADER_NAME"] = "FOO"
    app.config["JWT_REFRESH_CSRF_HEADER_NAME"] = "FOO"
    test_client = app.test_client()
    auth_url, csrf_cookie_name, post_url = options

    # Get the jwt cookies and csrf double submit tokens
    response = test_client.get(auth_url)
    csrf_token = _get_cookie_from_response(response, csrf_cookie_name)[csrf_cookie_name]

    # Test that you can post with the csrf double submit value
    csrf_headers = {"FOO": csrf_token}
    response = test_client.post(post_url, headers=csrf_headers)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


@pytest.mark.parametrize(
    "options",
    [
        ("/refresh_token", "csrf_refresh_token", "/post_refresh_protected"),
        ("/access_token", "csrf_access_token", "/post_protected"),
    ],
)
def test_csrf_with_default_form_field(app, options):
    app.config["JWT_CSRF_CHECK_FORM"] = True
    test_client = app.test_client()
    auth_url, csrf_cookie_name, post_url = options

    # Get the jwt cookies and csrf double submit tokens
    response = test_client.get(auth_url)
    csrf_token = _get_cookie_from_response(response, csrf_cookie_name)[csrf_cookie_name]

    # Test that you can post with the csrf double submit value
    csrf_data = {"csrf_token": csrf_token}
    response = test_client.post(post_url, data=csrf_data)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


@pytest.mark.parametrize(
    "options",
    [
        ("/refresh_token", "csrf_refresh_token", "/post_refresh_protected"),
        ("/access_token", "csrf_access_token", "/post_protected"),
    ],
)
def test_csrf_with_custom_form_field(app, options):
    app.config["JWT_CSRF_CHECK_FORM"] = True
    app.config["JWT_ACCESS_CSRF_FIELD_NAME"] = "FOO"
    app.config["JWT_REFRESH_CSRF_FIELD_NAME"] = "FOO"
    test_client = app.test_client()
    auth_url, csrf_cookie_name, post_url = options

    # Get the jwt cookies and csrf double submit tokens
    response = test_client.get(auth_url)
    csrf_token = _get_cookie_from_response(response, csrf_cookie_name)[csrf_cookie_name]

    # Test that you can post with the csrf double submit value
    csrf_data = {"FOO": csrf_token}
    response = test_client.post(post_url, data=csrf_data)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


@pytest.mark.parametrize(
    "options",
    [
        (
            "/refresh_token",
            "csrf_refresh_token",
            "/refresh_protected",
            "/post_refresh_protected",
        ),  # nopep8
        ("/access_token", "csrf_access_token", "/protected", "/post_protected"),
    ],
)
def test_custom_csrf_methods(app, options):
    app.config["JWT_CSRF_METHODS"] = ["GET"]
    test_client = app.test_client()
    auth_url, csrf_cookie_name, get_url, post_url = options

    # Get the jwt cookies and csrf double submit tokens
    response = test_client.get(auth_url)
    csrf_token = _get_cookie_from_response(response, csrf_cookie_name)[csrf_cookie_name]

    # Insure we can now do posts without csrf
    response = test_client.post(post_url)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}

    # Insure GET requests now fail without csrf
    response = test_client.get(get_url)
    assert response.status_code == 401
    assert response.get_json() == {"msg": "Missing CSRF token"}

    # Insure GET requests now succeed with csrf
    csrf_headers = {"X-CSRF-TOKEN": csrf_token}
    response = test_client.get(get_url, headers=csrf_headers)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


def test_default_cookie_options(app):
    test_client = app.test_client()

    # Test the default access cookies
    response = test_client.get("/access_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 2  # JWT and CSRF value

    access_cookie = _get_cookie_from_response(response, "access_token_cookie")
    assert access_cookie is not None
    assert access_cookie["path"] == "/"
    assert access_cookie["httponly"] is True
    assert "samesite" not in access_cookie

    access_csrf_cookie = _get_cookie_from_response(response, "csrf_access_token")
    assert access_csrf_cookie is not None
    assert access_csrf_cookie["path"] == "/"
    assert "httponly" not in access_csrf_cookie
    assert "samesite" not in access_csrf_cookie

    # Test the default refresh cookies
    response = test_client.get("/refresh_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 2  # JWT and CSRF value

    refresh_cookie = _get_cookie_from_response(response, "refresh_token_cookie")
    assert refresh_cookie is not None
    assert refresh_cookie["path"] == "/"
    assert refresh_cookie["httponly"] is True
    assert "samesite" not in refresh_cookie

    refresh_csrf_cookie = _get_cookie_from_response(response, "csrf_refresh_token")
    assert refresh_csrf_cookie is not None
    assert refresh_csrf_cookie["path"] == "/"
    assert "httponly" not in refresh_csrf_cookie
    assert "samesite" not in refresh_csrf_cookie


def test_custom_cookie_options(app):
    test_client = app.test_client()

    app.config["JWT_COOKIE_SECURE"] = True
    app.config["JWT_COOKIE_DOMAIN"] = "test.com"
    app.config["JWT_SESSION_COOKIE"] = False
    app.config["JWT_COOKIE_SAMESITE"] = "Strict"

    # Test access cookies with changed options
    response = test_client.get("/access_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 2  # JWT and CSRF value

    access_cookie = _get_cookie_from_response(response, "access_token_cookie")
    assert access_cookie is not None
    assert access_cookie["domain"] == "test.com"
    assert access_cookie["path"] == "/"
    assert access_cookie["expires"] != ""
    assert access_cookie["httponly"] is True
    assert access_cookie["secure"] is True
    assert access_cookie["samesite"] == "Strict"

    access_csrf_cookie = _get_cookie_from_response(response, "csrf_access_token")
    assert access_csrf_cookie is not None
    assert access_csrf_cookie["path"] == "/"
    assert access_csrf_cookie["secure"] is True
    assert access_csrf_cookie["domain"] == "test.com"
    assert access_csrf_cookie["expires"] != ""
    assert access_csrf_cookie["samesite"] == "Strict"

    # Test refresh cookies with changed options
    response = test_client.get("/refresh_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 2  # JWT and CSRF value

    refresh_cookie = _get_cookie_from_response(response, "refresh_token_cookie")
    assert refresh_cookie is not None
    assert refresh_cookie["domain"] == "test.com"
    assert refresh_cookie["path"] == "/"
    assert refresh_cookie["httponly"] is True
    assert refresh_cookie["secure"] is True
    assert refresh_cookie["expires"] != ""
    assert refresh_cookie["samesite"] == "Strict"

    refresh_csrf_cookie = _get_cookie_from_response(response, "csrf_refresh_token")
    assert refresh_csrf_cookie is not None
    assert refresh_csrf_cookie["path"] == "/"
    assert refresh_csrf_cookie["secure"] is True
    assert refresh_csrf_cookie["domain"] == "test.com"
    assert refresh_csrf_cookie["expires"] != ""
    assert refresh_csrf_cookie["samesite"] == "Strict"


def test_custom_cookie_names_and_paths(app):
    test_client = app.test_client()

    app.config["JWT_ACCESS_CSRF_COOKIE_NAME"] = "access_foo_csrf"
    app.config["JWT_REFRESH_CSRF_COOKIE_NAME"] = "refresh_foo_csrf"
    app.config["JWT_ACCESS_CSRF_COOKIE_PATH"] = "/protected"
    app.config["JWT_REFRESH_CSRF_COOKIE_PATH"] = "/refresh_protected"
    app.config["JWT_ACCESS_COOKIE_NAME"] = "access_foo"
    app.config["JWT_REFRESH_COOKIE_NAME"] = "refresh_foo"
    app.config["JWT_ACCESS_COOKIE_PATH"] = "/protected"
    app.config["JWT_REFRESH_COOKIE_PATH"] = "/refresh_protected"

    # Test the default access cookies
    response = test_client.get("/access_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 2  # JWT and CSRF value

    access_cookie = _get_cookie_from_response(response, "access_foo")
    access_csrf_cookie = _get_cookie_from_response(response, "access_foo_csrf")
    assert access_cookie is not None
    assert access_csrf_cookie is not None
    assert access_cookie["path"] == "/protected"
    assert access_csrf_cookie["path"] == "/protected"

    # Test the default refresh cookies
    response = test_client.get("/refresh_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 2  # JWT and CSRF value

    refresh_cookie = _get_cookie_from_response(response, "refresh_foo")
    refresh_csrf_cookie = _get_cookie_from_response(response, "refresh_foo_csrf")
    assert refresh_cookie is not None
    assert refresh_csrf_cookie is not None
    assert refresh_cookie["path"] == "/refresh_protected"
    assert refresh_csrf_cookie["path"] == "/refresh_protected"


def test_csrf_token_not_in_cookie(app):
    test_client = app.test_client()

    app.config["JWT_CSRF_IN_COOKIES"] = False

    # Test the default access cookies
    response = test_client.get("/access_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 1
    access_cookie = _get_cookie_from_response(response, "access_token_cookie")
    assert access_cookie is not None

    # Test the default refresh cookies
    response = test_client.get("/refresh_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 1
    refresh_cookie = _get_cookie_from_response(response, "refresh_token_cookie")
    assert refresh_cookie is not None


def test_cookies_without_csrf(app):
    test_client = app.test_client()

    app.config["JWT_COOKIE_CSRF_PROTECT"] = False

    # Test the default access cookies
    response = test_client.get("/access_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 1
    access_cookie = _get_cookie_from_response(response, "access_token_cookie")
    assert access_cookie is not None

    # Test the default refresh cookies
    response = test_client.get("/refresh_token")
    cookies = response.headers.getlist("Set-Cookie")
    assert len(cookies) == 1
    refresh_cookie = _get_cookie_from_response(response, "refresh_token_cookie")
    assert refresh_cookie is not None


def test_jwt_optional_with_csrf_enabled(app):
    test_client = app.test_client()

    # User without a token should be able to reach the endpoint without
    # getting a CSRF error
    response = test_client.post("/optional_post_protected")
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}

    # User with a token should still get a CSRF error if csrf not present
    response = test_client.get("/access_token")
    response = test_client.post("/optional_post_protected")
    assert response.status_code == 401
    assert response.get_json() == {"msg": "Missing CSRF token"}



import pytest
from flask import Flask
from flask import jsonify

from flask_jwt_extended import current_user
from flask_jwt_extended import get_current_user
from flask_jwt_extended import get_jwt
from flask_jwt_extended import get_jwt_header
from flask_jwt_extended import get_jwt_identity
from flask_jwt_extended import jwt_required
from flask_jwt_extended import JWTManager


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    jwt = JWTManager(app)

    @app.route("/optional", methods=["GET"])
    @jwt_required(optional=True)
    def access_protected():
        assert get_jwt() == {}
        assert get_jwt_header() == {}
        assert get_jwt_identity() == None  # noqa: E711
        assert get_current_user() == None  # noqa: E711
        assert current_user == None  # noqa: E711
        return jsonify(foo="bar")

    @jwt.user_lookup_loader
    def user_lookup_callback(_jwt_header, _jwt_data):
        assert True == False  # noqa: E712

    return app


def test_get_jwt_in_optional_route(app):
    test_client = app.test_client()
    response = test_client.get("/optional")
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}



from datetime import datetime
from datetime import timedelta
from datetime import timezone

import pytest
from dateutil.relativedelta import relativedelta
from flask import Flask
from jwt import DecodeError
from jwt import ExpiredSignatureError
from jwt import ImmatureSignatureError
from jwt import InvalidAudienceError
from jwt import InvalidIssuerError
from jwt import InvalidSignatureError
from jwt import MissingRequiredClaimError

from flask_jwt_extended import create_access_token
from flask_jwt_extended import create_refresh_token
from flask_jwt_extended import decode_token
from flask_jwt_extended import get_jti
from flask_jwt_extended import get_unverified_jwt_headers
from flask_jwt_extended import JWTManager
from flask_jwt_extended.config import config
from flask_jwt_extended.exceptions import JWTDecodeError
from tests.utils import encode_token
from tests.utils import get_jwt_manager


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "change_me"
    app.config["JWT_TOKEN_LOCATION"] = ["cookies", "headers"]
    app.config["JWT_COOKIE_CSRF_PROTECT"] = True
    JWTManager(app)
    return app


@pytest.fixture(scope="function")
def default_access_token(app):
    with app.test_request_context():
        return {
            "jti": "1234",
            config.identity_claim_key: "username",
            "type": "access",
            "fresh": True,
            "csrf": "abcd",
        }


@pytest.mark.parametrize("missing_claims", ["sub", "csrf"])
def test_missing_claims(app, default_access_token, missing_claims):
    del default_access_token[missing_claims]
    missing_jwt_token = encode_token(app, default_access_token)

    with pytest.raises(JWTDecodeError):
        with app.test_request_context():
            decode_token(missing_jwt_token, csrf_value="abcd")


def test_default_decode_token_values(app, default_access_token):
    del default_access_token["type"]
    del default_access_token["jti"]
    del default_access_token["fresh"]
    token = encode_token(app, default_access_token)

    with app.test_request_context():
        decoded = decode_token(token)
        assert decoded["type"] == "access"
        assert decoded["jti"] is None
        assert decoded["fresh"] is False


def test_supports_decoding_other_token_types(app, default_access_token):
    default_access_token["type"] = "app"
    other_token = encode_token(app, default_access_token)

    with app.test_request_context():
        decoded = decode_token(other_token)
        assert decoded["type"] == "app"


def test_encode_decode_audience(app):
    # Default, no audience
    with app.test_request_context():
        encoded_token = create_access_token("username")
        decoded_token = decode_token(encoded_token)
        with pytest.raises(KeyError):
            decoded_token["aud"]

    # Encode and decode audience configured
    app.config["JWT_ENCODE_AUDIENCE"] = "foo"
    app.config["JWT_DECODE_AUDIENCE"] = "foo"
    with app.test_request_context():
        encoded_token = create_access_token("username")
        decoded_token = decode_token(encoded_token)
        assert decoded_token["aud"] == "foo"

    # Encode and decode mismatch
    app.config["JWT_ENCODE_AUDIENCE"] = "foo"
    app.config["JWT_DECODE_AUDIENCE"] = "bar"
    with app.test_request_context():
        encoded_token = create_access_token("username")
        with pytest.raises(InvalidAudienceError):
            decode_token(encoded_token)

    # No encode defined
    app.config["JWT_ENCODE_AUDIENCE"] = None
    app.config["JWT_DECODE_AUDIENCE"] = "foo"
    with app.test_request_context():
        encoded_token = create_access_token("username")
        with pytest.raises(MissingRequiredClaimError):
            decode_token(encoded_token)

    # No decode defined
    app.config["JWT_ENCODE_AUDIENCE"] = "foo"
    app.config["JWT_DECODE_AUDIENCE"] = None
    with app.test_request_context():
        encoded_token = create_access_token("username")
        decoded_token = decode_token(encoded_token)
        assert decoded_token["aud"] == "foo"


@pytest.mark.parametrize("delta_func", [timedelta, relativedelta])
def test_expired_token(app, delta_func):
    with app.test_request_context():
        delta = delta_func(minutes=-5)
        access_token = create_access_token("username", expires_delta=delta)
        refresh_token = create_refresh_token("username", expires_delta=delta)
        with pytest.raises(ExpiredSignatureError):
            decode_token(access_token)
        with pytest.raises(ExpiredSignatureError):
            decode_token(refresh_token)


@pytest.mark.parametrize("delta_func", [timedelta, relativedelta])
def test_allow_expired_token(app, delta_func):
    with app.test_request_context():
        delta = delta_func(minutes=-5)
        access_token = create_access_token("username", expires_delta=delta)
        refresh_token = create_refresh_token("username", expires_delta=delta)
        for token in (access_token, refresh_token):
            decoded = decode_token(token, allow_expired=True)
            assert decoded["sub"] == "username"
            assert "exp" in decoded


def test_never_expire_token(app):
    with app.test_request_context():
        access_token = create_access_token("username", expires_delta=False)
        refresh_token = create_refresh_token("username", expires_delta=False)
        for token in (access_token, refresh_token):
            decoded = decode_token(token)
            assert "exp" not in decoded


def test_nbf_token_in_future(app):
    date_in_future = datetime.utcnow() + timedelta(seconds=30)

    with pytest.raises(ImmatureSignatureError):
        with app.test_request_context():
            access_token = create_access_token(
                "username", additional_claims={"nbf": date_in_future}
            )
            decode_token(access_token)

    with app.test_request_context():
        app.config["JWT_DECODE_LEEWAY"] = 30
        access_token = create_access_token("username")
        decode_token(access_token)


def test_alternate_identity_claim(app, default_access_token):
    app.config["JWT_IDENTITY_CLAIM"] = "banana"

    # Insure decoding fails if the claim isn't there
    token = encode_token(app, default_access_token)
    with pytest.raises(JWTDecodeError):
        with app.test_request_context():
            decode_token(token)

    # Insure the claim exists in the decoded jwt
    del default_access_token["sub"]
    default_access_token["banana"] = "username"
    token = encode_token(app, default_access_token)
    with app.test_request_context():
        decoded = decode_token(token)
        assert "banana" in decoded
        assert "sub" not in decoded


def test_get_jti(app, default_access_token):
    token = encode_token(app, default_access_token)

    with app.test_request_context():
        assert default_access_token["jti"] == get_jti(token)


def test_encode_decode_callback_values(app, default_access_token):
    jwtM = get_jwt_manager(app)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    with app.test_request_context():
        assert jwtM._decode_key_callback({}, {}) == "foobarbaz"
        assert jwtM._encode_key_callback({}) == "foobarbaz"

    @jwtM.encode_key_loader
    def get_encode_key_1(identity):
        return "different secret"

    assert jwtM._encode_key_callback("") == "different secret"

    @jwtM.decode_key_loader
    def get_decode_key_1(jwt_header, jwt_data):
        return "different secret"

    assert jwtM._decode_key_callback({}, {}) == "different secret"


def test_custom_encode_decode_key_callbacks(app, default_access_token):
    jwtM = get_jwt_manager(app)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"

    @jwtM.encode_key_loader
    def get_encode_key_1(identity):
        assert identity == "username"
        return "different secret"

    with pytest.raises(InvalidSignatureError):
        with app.test_request_context():
            token = create_access_token("username")
            decode_token(token)
    with pytest.raises(InvalidSignatureError):
        with app.test_request_context():
            token = create_refresh_token("username")
            decode_token(token)

    @jwtM.decode_key_loader
    def get_decode_key_1(jwt_header, jwt_data):
        assert jwt_header["alg"] == "HS256"
        assert jwt_data["sub"] == "username"
        return "different secret"

    with app.test_request_context():
        token = create_access_token("username")
        decode_token(token)
        token = create_refresh_token("username")
        decode_token(token)


@pytest.mark.parametrize("token_aud", ["foo", ["bar"], ["foo", "bar", "baz"]])
def test_valid_aud(app, default_access_token, token_aud):
    app.config["JWT_DECODE_AUDIENCE"] = ["foo", "bar"]

    default_access_token["aud"] = token_aud
    valid_token = encode_token(app, default_access_token)
    with app.test_request_context():
        decoded = decode_token(valid_token)
        assert decoded["aud"] == token_aud


@pytest.mark.parametrize("token_aud", ["bar", ["bar"], ["bar", "baz"]])
def test_invalid_aud(app, default_access_token, token_aud):
    app.config["JWT_DECODE_AUDIENCE"] = "foo"

    default_access_token["aud"] = token_aud
    invalid_token = encode_token(app, default_access_token)
    with pytest.raises(InvalidAudienceError):
        with app.test_request_context():
            decode_token(invalid_token)


@pytest.mark.parametrize("token_aud", ["bar", ["bar"], ["bar", "baz"]])
def test_verify_no_aud(app, default_access_token, token_aud):
    app.config["JWT_DECODE_AUDIENCE"] = None

    default_access_token["aud"] = token_aud
    valid_token = encode_token(app, default_access_token)
    with app.test_request_context():
        decoded = decode_token(valid_token)
        assert decoded["aud"] == token_aud


def test_encode_iss(app, default_access_token):
    app.config["JWT_ENCODE_ISSUER"] = "foobar"

    with app.test_request_context():
        access_token = create_access_token("username")
        decoded = decode_token(access_token)
        assert decoded["iss"] == "foobar"


def test_mismatch_iss(app, default_access_token):
    app.config["JWT_ENCODE_ISSUER"] = "foobar"
    app.config["JWT_DECODE_ISSUER"] = "baz"

    with pytest.raises(InvalidIssuerError):
        with app.test_request_context():
            invalid_token = create_access_token("username")
            decode_token(invalid_token)


def test_valid_decode_iss(app, default_access_token):
    app.config["JWT_DECODE_ISSUER"] = "foobar"

    default_access_token["iss"] = "foobar"
    valid_token = encode_token(app, default_access_token)
    with app.test_request_context():
        decoded = decode_token(valid_token)
        assert decoded["iss"] == "foobar"


def test_invalid_decode_iss(app, default_access_token):
    app.config["JWT_DECODE_ISSUER"] = "baz"

    default_access_token["iss"] = "foobar"
    invalid_token = encode_token(app, default_access_token)
    with pytest.raises(InvalidIssuerError):
        with app.test_request_context():
            decode_token(invalid_token)


def test_malformed_token(app):
    invalid_token = "foobarbaz"
    with pytest.raises(DecodeError):
        with app.test_request_context():
            decode_token(invalid_token)


def test_jwt_headers(app):
    jwt_header = {"foo": "bar"}
    with app.test_request_context():
        access_token = create_access_token("username", additional_headers=jwt_header)
        refresh_token = create_refresh_token("username", additional_headers=jwt_header)
        assert get_unverified_jwt_headers(access_token)["foo"] == "bar"
        assert get_unverified_jwt_headers(refresh_token)["foo"] == "bar"


def test_token_expires_time(app):
    app.config["JWT_ACCESS_TOKEN_EXPIRES"] = timedelta(hours=1)
    app.config["JWT_REFRESH_TOKEN_EXPIRES"] = timedelta(hours=2)

    now_timestamp = datetime.timestamp(datetime.now(timezone.utc))

    with app.test_request_context():
        access_token = create_access_token("username")
        refresh_token = create_refresh_token("username")
        access_timestamp = decode_token(access_token)["exp"]
        refresh_timestamp = decode_token(refresh_token)["exp"]

        # <  2 for a little bit of leeway from when we calculated now vs when
        # the tokens are created
        assert (access_timestamp - (now_timestamp + 3600)) < 2
        assert (refresh_timestamp - (now_timestamp + 7200)) < 2


def test_nbf_is_present_by_default(app):
    with app.test_request_context():
        access_token = create_access_token("username", fresh=True)
        decoded = decode_token(access_token)
        assert "nbf" in decoded


def test_disable_nbf_encoding(app):
    app.config["JWT_ENCODE_NBF"] = False
    with app.test_request_context():
        access_token = create_access_token("username", fresh=True)
        decoded = decode_token(access_token)
        assert "nbf" not in decoded



import pytest
from flask import Flask
from flask import jsonify

from flask_jwt_extended import create_access_token
from flask_jwt_extended import create_refresh_token
from flask_jwt_extended import jwt_required
from flask_jwt_extended import JWTManager
from tests.utils import get_jwt_manager


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    app.config["JWT_SECRET_KEY"] = "foobarbaz"
    app.config["JWT_TOKEN_LOCATION"] = "json"
    JWTManager(app)

    @app.route("/protected", methods=["POST"])
    @jwt_required()
    def access_protected():
        return jsonify(foo="bar")

    @app.route("/refresh", methods=["POST"])
    @jwt_required(refresh=True)
    def refresh_protected():
        return jsonify(foo="bar")

    return app


def test_content_type(app):
    test_client = app.test_client()

    with app.test_request_context():
        access_token = create_access_token("username")
        refresh_token = create_refresh_token("username")

    data = {"access_token": access_token}
    response = test_client.post("/protected", data=data)
    expected_json = {"msg": "Invalid content-type. Must be application/json."}
    assert response.status_code == 401
    assert response.get_json() == expected_json

    data = {"refresh_token": refresh_token}
    response = test_client.post("/refresh", data=data)
    expected_json = {"msg": "Invalid content-type. Must be application/json."}
    assert response.status_code == 401
    assert response.get_json() == expected_json


def test_custom_body_key(app):
    app.config["JWT_JSON_KEY"] = "Foo"
    app.config["JWT_REFRESH_JSON_KEY"] = "Bar"
    test_client = app.test_client()

    with app.test_request_context():
        access_token = create_access_token("username")
        refresh_token = create_refresh_token("username")

    # Ensure 'default' keys no longer work
    data = {"access_token": access_token}
    response = test_client.post("/protected", json=data)
    assert response.status_code == 401
    assert response.get_json() == {"msg": 'Missing "Foo" key in json data.'}

    data = {"refresh_token": refresh_token}
    response = test_client.post("/refresh", json=data)
    assert response.status_code == 401
    assert response.get_json() == {"msg": 'Missing "Bar" key in json data.'}

    # Ensure new keys do work
    data = {"Foo": access_token}
    response = test_client.post("/protected", json=data)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}

    data = {"Bar": refresh_token}
    response = test_client.post("/refresh", json=data)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


def test_missing_keys(app):
    test_client = app.test_client()
    jwtM = get_jwt_manager(app)
    headers = {"content-type": "application/json"}

    # Ensure 'default' no json response
    response = test_client.post("/protected", headers=headers)
    assert response.status_code == 401
    assert response.get_json() == {"msg": 'Missing "access_token" key in json data.'}

    # Test custom no json response
    @jwtM.unauthorized_loader
    def custom_response(err_str):
        return jsonify(foo="bar"), 201

    response = test_client.post("/protected", headers=headers)
    assert response.status_code == 201
    assert response.get_json() == {"foo": "bar"}


def test_defaults(app):
    test_client = app.test_client()

    with app.test_request_context():
        access_token = create_access_token("username")
        refresh_token = create_refresh_token("username")

    data = {"access_token": access_token}
    response = test_client.post("/protected", json=data)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}

    data = {"refresh_token": refresh_token}
    response = test_client.post("/refresh", json=data)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}


def test_custom_content_type(app):
    test_client = app.test_client()
    content_type = "application/json;charset=UTF-8"

    with app.test_request_context():
        access_token = create_access_token("username")
        refresh_token = create_refresh_token("username")

    data = {"access_token": access_token}
    response = test_client.post("/protected", json=data, content_type=content_type)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}

    data = {"refresh_token": refresh_token}
    response = test_client.post("/refresh", json=data, content_type=content_type)
    assert response.status_code == 200
    assert response.get_json() == {"foo": "bar"}



import pytest
from flask import Flask

from flask_jwt_extended import current_user
from flask_jwt_extended import get_current_user
from flask_jwt_extended import get_jwt
from flask_jwt_extended import get_jwt_header
from flask_jwt_extended import get_jwt_identity
from flask_jwt_extended import JWTManager


@pytest.fixture(scope="function")
def app():
    app = Flask(__name__)
    JWTManager(app)
    return app


def test_get_jwt_in_non_protected_route(app):
    with app.test_request_context():
        with pytest.raises(RuntimeError):
            get_jwt()


def test_get_jwt_header_in_non_protected_route(app):
    with app.test_request_context():
        with pytest.raises(RuntimeError):
            get_jwt_header()


def test_get_jwt_identity_in_non_protected_route(app):
    with app.test_request_context():
        with pytest.raises(RuntimeError):
            get_jwt_identity()


def test_current_user_in_non_protected_route(app):
    with app.test_request_context():
        with pytest.raises(RuntimeError):
            current_user.foo


def test_get_current_user_in_non_protected_route(app):
    with app.test_request_context():
        with pytest.raises(RuntimeError):
            get_current_user()



import io
import re

from setuptools import setup

with io.open("flask_jwt_extended/__init__.py", encoding="utf-8") as f:
    version = re.search(r"__version__ = \"(.+)\"", f.read()).group(1)


with open("README.md", "r") as f:
    long_description = f.read()


setup(
    name="Flask-JWT-Extended",
    version=version,
    url="https://github.com/vimalloc/flask-jwt-extended",
    license="MIT",
    author="Landon Gilbert-Bland",
    author_email="landongb@hey.com",
    description="Extended JWT integration with Flask",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=["flask", "jwt", "json web token"],
    packages=["flask_jwt_extended"],
    zip_safe=False,
    platforms="any",
    install_requires=[
        "Werkzeug>=0.14",  # Needed for SameSite cookie functionality
        "Flask>=1.0,<3.0",
        "PyJWT>=2.0,<3.0",
    ],
    extras_require={"asymmetric_crypto": ["cryptography>=3.0,<4.0"]},
    python_requires=">=3.6,<4",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Flask",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



from .jwt_manager import JWTManager
from .utils import create_access_token
from .utils import create_refresh_token
from .utils import current_user
from .utils import decode_token
from .utils import get_csrf_token
from .utils import get_current_user
from .utils import get_jti
from .utils import get_jwt
from .utils import get_jwt_header
from .utils import get_jwt_identity
from .utils import get_jwt_request_location
from .utils import get_unverified_jwt_headers
from .utils import set_access_cookies
from .utils import set_refresh_cookies
from .utils import unset_access_cookies
from .utils import unset_jwt_cookies
from .utils import unset_refresh_cookies
from .view_decorators import jwt_required
from .view_decorators import verify_jwt_in_request

__version__ = "4.2.2"
